@@ -9,6 +9,10 @@ from pyinfra.api import FactBase
 from .util.packaging import parse_packages
 
 
+DEB_PACKAGE_NAME_REGEX = r'[a-zA-Z0-9\+\-\.]+'
+DEB_PACKAGE_VERSION_REGEX = r'[a-zA-Z0-9:~\.\-\+]+'
+
+
 class DebPackages(FactBase):
     '''
     Returns a dict of installed dpkg packages:
@@ -25,7 +29,10 @@ class DebPackages(FactBase):
 
     default = dict
 
-    regex = r'^ii\s+([a-zA-Z0-9\+\-\.]+):?[a-zA-Z0-9]*\s+([a-zA-Z0-9:~\.\-\+]+).+$'
+    regex = r'^[i|h]i\s+({0}):?[a-zA-Z0-9]*\s+({1}).+$'.format(
+        DEB_PACKAGE_NAME_REGEX,
+        DEB_PACKAGE_VERSION_REGEX,
+    )
 
     def process(self, output):
         return parse_packages(self.regex, output)
@@ -37,8 +44,8 @@ class DebPackage(FactBase):
     '''
 
     _regexes = {
-        'name': r'^Package: ([a-zA-Z0-9\-]+)$',
-        'version': r'^Version: ([0-9\:\.\-]+)$',
+        'name': r'^Package: ({0})$'.format(DEB_PACKAGE_NAME_REGEX),
+        'version': r'^Version: ({0})$'.format(DEB_PACKAGE_VERSION_REGEX),
     }
 
     requires_command = 'dpkg'
