@@ -91,7 +91,9 @@ class Arch(FactBase):
     Returns the system architecture according to ``uname``.
     '''
 
-    command = 'uname -p'
+    # ``uname -p`` is not portable and returns ``unknown`` on Debian.
+    # ``uname -m`` works on most Linux and BSD systems.
+    command = 'uname -m'
 
 
 class Command(FactBase):
@@ -422,6 +424,7 @@ class Users(FactBase):
                 ],
                 'uid': user_id,
                 'gid': main_user_group_id,
+                'lastlog': last_login_time,
             },
         }
     '''
@@ -429,7 +432,8 @@ class Users(FactBase):
     command = '''
         for i in `cat /etc/passwd | cut -d: -f1`; do
             ENTRY=`grep ^$i: /etc/passwd`;
-            echo "$ENTRY|`id -gn $i`|`id -Gn $i`";
+            LASTLOG=`lastlog -u $i | grep ^$i` | tr -s ' ';
+            echo "$ENTRY|`id -gn $i`|`id -Gn $i`|$LASTLOG";
         done
     '''.strip()
 
@@ -437,9 +441,10 @@ class Users(FactBase):
 
     def process(self, output):
         users = {}
+        rex = r'[A-Z][a-z]{2} [A-Z][a-z]{2} {1,2}\d+ .+$'
 
         for line in output:
-            entry, group, user_groups = line.split('|')
+            entry, group, user_groups, lastlog = line.split('|')
 
             if entry:
                 # Parse out the comment/home/shell
@@ -452,6 +457,16 @@ class Users(FactBase):
                     if group_name and group_name != group:
                         groups.append(group_name)
 
+                raw_login_time = None
+                login_time = None
+
+                # Parse lastlog info
+                # lastlog output varies, which is why I use regex to match login time
+                login = re.search(rex, lastlog)
+                if login:
+                    raw_login_time = login.group()
+                    login_time = parse_date(raw_login_time)
+
                 users[entries[0]] = {
                     'home': entries[5] or None,
                     'comment': entries[4] or None,
@@ -460,6 +475,8 @@ class Users(FactBase):
                     'groups': groups,
                     'uid': int(entries[2]),
                     'gid': int(entries[3]),
+                    'lastlog': raw_login_time,
+                    'login_time': login_time,
                 }
 
         return users
